@@ -9,6 +9,7 @@ use Mojo::Template;
 use Mojo::JSON qw/encode_json decode_json/;
 use SReview::Config::Common;
 use SReview::Talk::State;
+use SReview::Talk::Progress;
 use DateTime::Format::Pg;
 
 my $config = SReview::Config::Common::setup;
@@ -60,6 +61,17 @@ has 'talkid' => (
         },
 );
 
+has 'upstreamid' => (
+	is => 'ro',
+	isa => 'Str',
+	builder => '_probe_upstreamid',
+	lazy => 1,
+);
+
+sub _probe_upstreamid {
+	return shift->_get_pathinfo->{raw}{upstreamid}
+}
+
 =head2 pathinfo
 
 Helper property to look up information from the database. Should not be
@@ -79,7 +91,7 @@ sub _load_pathinfo {
 
 	my $pathinfo = {};
 
-	my $eventdata = $pg->db->dbh->prepare("SELECT events.id AS eventid, events.name AS event, events.outputdir AS event_output, rooms.name AS room, rooms.outputname AS room_output, rooms.id AS room_id, talks.starttime, talks.starttime::date AS date, to_char(starttime, 'DD Month yyyy at HH:MI') AS readable_date, to_char(talks.starttime, 'yyyy') AS year, talks.endtime, talks.slug, talks.title, talks.subtitle, talks.state, talks.nonce, talks.apologynote FROM talks JOIN events ON talks.event = events.id JOIN rooms ON rooms.id = talks.room WHERE talks.id = ?");
+	my $eventdata = $pg->db->dbh->prepare("SELECT events.id AS eventid, events.name AS event, events.outputdir AS event_output, rooms.name AS room, rooms.outputname AS room_output, rooms.id AS room_id, talks.starttime, talks.starttime::date AS date, to_char(starttime, 'DD Month yyyy at HH:MI') AS readable_date, to_char(talks.starttime, 'yyyy') AS year, talks.endtime, talks.slug, talks.title, talks.subtitle, talks.state, talks.progress, talks.nonce, talks.apologynote, talks.upstreamid FROM talks JOIN events ON talks.event = events.id JOIN rooms ON rooms.id = talks.room WHERE talks.id = ?");
 	$eventdata->execute($self->talkid);
 	my $row = $eventdata->fetchrow_hashref();
 
@@ -183,6 +195,29 @@ sub _load_comment {
         return $row->{comments};
 }
 
+=head2 first_comment
+
+The most recent comment entered in the "other brokenness" field.
+
+=cut
+
+has 'first_comment' => (
+	lazy => 1,
+	is => 'rw',
+	builder => '_load_first_comment',
+	clearer => 'clear_first_comment',
+	predicate => 'has_first_comment',
+);
+
+sub _load_first_comment {
+	my $self = shift;
+
+	my $st = $pg->db->dbh->prepare("WITH orderedlog(talk, comment) AS (SELECT talk, comment FROM commentlog ORDER BY logdate DESC) SELECT talk, logdate FROM orderedlog WHERE talk = ? LIMIT 1");
+	$st->execute($self->talkid);
+	my $row = $st->fetchrow_hashref;
+	return $row->{comment};
+}
+
 =head2 corrected_times
 
 The start- and endtime of the talk, with corrections (if any) applied.
@@ -308,7 +343,32 @@ has 'event_output' => (
 );
 
 sub _load_event_output {
-	return shift->_get_pathinfo->{raw}{event_output};
+	my $self = shift;
+
+	my $rv = $self->_get_pathinfo->{raw}{event_output};
+	if(!defined($rv) || length($rv) == 0) {
+		$rv = $self->_get_pathinfo->{raw}{event};
+		$rv =~ s/[^a-zA-Z0-9]/-/g;
+	}
+	return $rv;
+}
+
+=head2 progress
+
+The current progress value of the talk, as an L<SReview::Talk::Progress>
+
+=cut
+
+has 'progress' => (
+	lazy => 1,
+	is => 'rw',
+	isa => 'SReview::Talk::Progress',
+	builder => '_load_progress',
+);
+
+sub _load_progress {
+	my $self = shift;
+	return SReview::Talk::Progress->new($self->_get_pathinfo->{raw}{progress});
 }
 
 =head2 state
@@ -573,7 +633,7 @@ sub _load_video_fragments {
 	my $talk_data = $pg->db->dbh->prepare("SELECT talkid, rawid, raw_filename, extract(epoch from fragment_start) AS fragment_start, extract(epoch from raw_length) as raw_length, extract(epoch from raw_length_corrected) as raw_length_corrected FROM adjusted_raw_talks(?, make_interval(secs :=?::numeric), make_interval(secs := ?::numeric)) ORDER BY talk_start, raw_start");
 	$talk_data->execute($self->talkid, $corrections->{"offset_start"}, $corrections->{"length_adj"});
 
-	my $rows;
+	my $rows = [];
 	while(my $row = $talk_data->fetchrow_hashref()) {
 		push @$rows, $row;
 	}
@@ -724,6 +784,7 @@ sub _load_eventurl {
 			event => $self->eventname,
 			event_output => $self->event_output,
 			talk => $self,
+			room_output => $self->_get_pathinfo->{raw}{room_output} // $self->_get_pathinfo->{raw}{room},
 			year => $self->_get_pathinfo->{raw}{year}});
 		chomp $rv;
 		return $rv;
